=== modified file 'lib/canonical/launchpad/security.py'
--- lib/canonical/launchpad/security.py	2011-09-06 14:03:34 +0000
+++ lib/canonical/launchpad/security.py	2011-09-07 12:59:27 +0000
@@ -38,7 +38,7 @@
 from lp.app.security import (
     AnonymousAuthorization,
     AuthorizationBase,
-    ForwardedAuthorization,
+    DelegatedAuthorization,
     )
 from lp.archivepublisher.interfaces.publisherconfig import IPublisherConfig
 from lp.blueprints.interfaces.specification import (
@@ -1003,7 +1003,7 @@
             archive.getComponentsForQueueAdmin(user.person)) or user.in_admin
 
 
-class EditDistroSeriesDifference(ForwardedAuthorization):
+class EditDistroSeriesDifference(DelegatedAuthorization):
     """Anyone with lp.View on the distribution can edit a DSD."""
     permission = 'launchpad.Edit'
     usedfor = IDistroSeriesDifferenceEdit
@@ -1929,7 +1929,7 @@
                 self.forwardCheckAuthenticated(user, self.obj.distribution))
 
 
-class AdminDistributionSourcePackageTranslations(ForwardedAuthorization):
+class AdminDistributionSourcePackageTranslations(DelegatedAuthorization):
     """DistributionSourcePackage objects link to a distribution."""
     permission = 'launchpad.TranslationsAdmin'
     usedfor = IDistributionSourcePackage
@@ -1982,7 +1982,7 @@
             self.branches))
 
 
-class PreviewDiffView(ForwardedAuthorization):
+class PreviewDiffView(DelegatedAuthorization):
     permission = 'launchpad.View'
     usedfor = IPreviewDiff
 
@@ -1990,7 +1990,7 @@
         super(PreviewDiffView, self).__init__(obj.branch_merge_proposal)
 
 
-class CodeReviewVoteReferenceEdit(ForwardedAuthorization):
+class CodeReviewVoteReferenceEdit(DelegatedAuthorization):
     permission = 'launchpad.Edit'
     usedfor = ICodeReviewVoteReference
 
@@ -2015,7 +2015,7 @@
                     user))
 
 
-class CodeReviewCommentView(ForwardedAuthorization):
+class CodeReviewCommentView(DelegatedAuthorization):
     permission = 'launchpad.View'
     usedfor = ICodeReviewComment
 
@@ -2024,7 +2024,7 @@
             obj.branch_merge_proposal)
 
 
-class CodeReviewCommentDelete(ForwardedAuthorization):
+class CodeReviewCommentDelete(DelegatedAuthorization):
     permission = 'launchpad.Edit'
     usedfor = ICodeReviewCommentDeletion
 
@@ -2291,7 +2291,7 @@
         return auth_edit.checkAuthenticated(user)
 
 
-class EditArchiveAuthToken(ForwardedAuthorization):
+class EditArchiveAuthToken(DelegatedAuthorization):
     """Restrict editing of archive tokens.
 
     The user should have append privileges to the context archive, or be an
@@ -2309,7 +2309,7 @@
                 super(EditArchiveAuthToken, self).checkAuthenticated(user))
 
 
-class ViewPersonalArchiveSubscription(ForwardedAuthorization):
+class ViewPersonalArchiveSubscription(DelegatedAuthorization):
     """Restrict viewing of personal archive subscriptions (non-db class).
 
     The user should be the subscriber, have append privilege to the archive
@@ -2330,7 +2330,7 @@
             ViewPersonalArchiveSubscription, self).checkAuthenticated(user)
 
 
-class ViewArchiveSubscriber(ForwardedAuthorization):
+class ViewArchiveSubscriber(DelegatedAuthorization):
     """Restrict viewing of archive subscribers.
 
     The user should be the subscriber, have append privilege to the
@@ -2349,7 +2349,7 @@
                 super(ViewArchiveSubscriber, self).checkAuthenticated(user))
 
 
-class EditArchiveSubscriber(ForwardedAuthorization):
+class EditArchiveSubscriber(DelegatedAuthorization):
     """Restrict editing of archive subscribers.
 
     The user should have append privilege to the archive or be an admin.
@@ -2366,34 +2366,7 @@
                 super(EditArchiveSubscriber, self).checkAuthenticated(user))
 
 
-class DerivedAuthorization(AuthorizationBase):
-    """An Authorization that is based on permissions for other objects.
-
-    Implementations must define permission, usedfor and iter_objects.
-    iter_objects should iterate through the objects to check permission on.
-
-    Failure on the permission check for any object causes an overall failure.
-    """
-
-    def iter_adapters(self):
-        return (
-            getAdapter(obj, IAuthorization, self.permission)
-            for obj in self.iter_objects())
-
-    def checkAuthenticated(self, user):
-        for adapter in self.iter_adapters():
-            if not adapter.checkAuthenticated(user):
-                return False
-        return True
-
-    def checkUnauthenticated(self):
-        for adapter in self.iter_adapters():
-            if not adapter.checkUnauthenticated():
-                return False
-        return True
-
-
-class ViewSourcePackageRecipe(DerivedAuthorization):
+class ViewSourcePackageRecipe(DelegatedAuthorization):
 
     permission = "launchpad.View"
     usedfor = ISourcePackageRecipe
@@ -2402,7 +2375,7 @@
         return self.obj.getReferencedBranches()
 
 
-class ViewSourcePackageRecipeBuild(DerivedAuthorization):
+class ViewSourcePackageRecipeBuild(DelegatedAuthorization):
 
     permission = "launchpad.View"
     usedfor = ISourcePackageRecipeBuild
@@ -2422,7 +2395,7 @@
         super(ViewSourcePackagePublishingHistory, self).__init__(obj.archive)
 
 
-class EditPublishing(ForwardedAuthorization):
+class EditPublishing(DelegatedAuthorization):
     """Restrict editing of source and binary packages.."""
     permission = "launchpad.Edit"
     usedfor = IPublishingEdit

=== modified file 'lib/lp/app/security.py'
--- lib/lp/app/security.py	2011-09-02 18:06:22 +0000
+++ lib/lp/app/security.py	2011-09-07 12:59:27 +0000
@@ -8,7 +8,7 @@
 __all__ = [
     'AnonymousAuthorization',
     'AuthorizationBase',
-    'ForwardedAuthorization',
+    'DelegatedAuthorization',
     ]
 
 from zope.component import queryAdapter
@@ -107,29 +107,38 @@
         return True
 
 
-class ForwardedAuthorization(AuthorizationBase):
-
-    permission = None
-    usedfor = None
-
-    def __init__(self, forwarded_object, permission=None):
+class DelegatedAuthorization(AuthorizationBase):
+
+    def __init__(self, forwarded_object=None, permission=None):
         self.forwarded_object = forwarded_object
         if permission is not None:
             self.permission = permission
 
-    @property
-    def forwarded_adapter(self):
-        return queryAdapter(
-            self.forwarded_object, IAuthorization, self.permission)
+    def iter_objects(self):
+        """Iterator of objects used for authentication checking.
+
+        If an object is provided when the class is instantiated, it will be
+        used.  Otherwise this method must be overridden to provide the objects
+        to be used.
+        """
+        if self.forwarded_object is None:
+            raise ValueError(
+                "Either set forwarded_object or override iter_objects.")
+        yield self.forwarded_object
+
+    def iter_adapters(self):
+        return (
+            queryAdapter(obj, IAuthorization, self.permission)
+            for obj in self.iter_objects())
 
     def checkAuthenticated(self, user):
-        adapter = self.forwarded_adapter
-        if adapter is None:
-            return False
-        return adapter.checkAuthenticated(user)
+        for adapter in self.iter_adapters():
+            if adapter is None or not adapter.checkAuthenticated(user):
+                return False
+        return True
 
     def checkUnauthenticated(self):
-        adapter = self.forwarded_adapter
-        if adapter is None:
-            return False
-        return adapter.checkUnauthenticated()
+        for adapter in self.iter_adapters():
+            if adapter is None or not adapter.checkUnauthenticated():
+                return False
+        return True

=== modified file 'lib/lp/app/tests/test_security.py'
--- lib/lp/app/tests/test_security.py	2011-09-05 15:17:26 +0000
+++ lib/lp/app/tests/test_security.py	2011-09-07 12:59:27 +0000
@@ -16,7 +16,7 @@
 from lp.app.interfaces.security import IAuthorization
 from lp.app.security import (
     AuthorizationBase,
-    ForwardedAuthorization,
+    DelegatedAuthorization,
     )
 from lp.testing import TestCaseWithFactory
 from lp.testing.fakemethod import FakeMethod
@@ -145,9 +145,9 @@
             adapter.forwardCheckAuthenticated(None, Dummy()))
 
 
-class FakeForwardedAuthorization(ForwardedAuthorization):
+class FakeDelegatedAuthorization(DelegatedAuthorization):
     def __init__(self, obj, permission=None):
-        super(FakeForwardedAuthorization, self).__init__(
+        super(FakeDelegatedAuthorization, self).__init__(
             obj.child_obj, permission)
 
 
@@ -158,15 +158,15 @@
         self.child_obj = Dummy()
 
 
-class TestForwardedAuthorizationBase(TestCaseWithFactory):
+class TestDelegatedAuthorizationBase(TestCaseWithFactory):
 
     layer = ZopelessDatabaseLayer
 
-    def test_ForwardedAuthorization_same_permissions(self):
+    def test_DelegatedAuthorization_same_permissions(self):
 
         permission = self.factory.getUniqueString()
         fake_obj = FakeForwardedObject()
-        outer_adapter = FakeForwardedAuthorization(fake_obj)
+        outer_adapter = FakeDelegatedAuthorization(fake_obj)
         outer_adapter.permission = permission
 
         inner_adapter = FakeSecurityAdapter()
@@ -179,11 +179,11 @@
             (1, inner_adapter.checkAuthenticated.call_count),
             (1, inner_adapter.checkUnauthenticated.call_count))
 
-    def test_ForwardedAuthorization_different_permissions(self):
+    def test_DelegatedAuthorization_different_permissions(self):
         perm_inner = 'inner'
         perm_outer = 'outer'
         fake_obj = FakeForwardedObject()
-        outer_adapter = FakeForwardedAuthorization(fake_obj, perm_inner)
+        outer_adapter = FakeDelegatedAuthorization(fake_obj, perm_inner)
         registerFakeSecurityAdapter(IDummy, perm_outer, outer_adapter)
 
         inner_adapter = FakeSecurityAdapter()

=== modified file 'lib/lp/bugs/security.py'
--- lib/lp/bugs/security.py	2011-09-01 20:07:54 +0000
+++ lib/lp/bugs/security.py	2011-09-07 12:59:27 +0000
@@ -11,7 +11,7 @@
 from lp.app.security import (
     AnonymousAuthorization,
     AuthorizationBase,
-    ForwardedAuthorization,
+    DelegatedAuthorization,
     )
 from lp.bugs.interfaces.bug import IBug
 from lp.bugs.interfaces.bugattachment import IBugAttachment
@@ -101,7 +101,7 @@
         super(EditBugBranch, self).__init__(bug_branch.bug)
 
 
-class ViewBugAttachment(ForwardedAuthorization):
+class ViewBugAttachment(DelegatedAuthorization):
     """Security adapter for viewing a bug attachment.
 
     If the user is authorized to view the bug, he's allowed to view the
@@ -114,7 +114,7 @@
         super(ViewBugAttachment, self).__init__(bugattachment.bug)
 
 
-class EditBugAttachment(ForwardedAuthorization):
+class EditBugAttachment(DelegatedAuthorization):
     """Security adapter for editing a bug attachment.
 
     If the user is authorized to view the bug, he's allowed to edit the

